# Copyright 2016-2017 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

FWG = 'firewall_group'
FWGS = 'firewall_groups'
FWP = 'firewall_policy'
FWPS = 'firewall_policies'
FWR = 'firewall_rule'
FWRS = 'firewall_rules'
CMD_FWG = 'fwaas_' + FWG
CMD_FWP = 'fwaas_' + FWP
CMD_FWR = 'fwaas_' + FWR
