# Copyright (c) 2016 Juniper Networks Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

BGPVPN = 'bgpvpn'
BGPVPNS = '%ss' % BGPVPN

NETWORK_RESOURCE_NAME = 'network'
NETWORK_ASSOC = '%s_association' % NETWORK_RESOURCE_NAME
NETWORK_ASSOCS = '%ss' % NETWORK_ASSOC

ROUTER_RESOURCE_NAME = 'router'
ROUTER_ASSOC = '%s_association' % ROUTER_RESOURCE_NAME
ROUTER_ASSOCS = '%ss' % ROUTER_ASSOC

PORT_RESOURCE_NAME = 'port'
PORT_ASSOC = '%s_association' % PORT_RESOURCE_NAME
PORT_ASSOCS = '%ss' % PORT_ASSOC
